/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM                                   |
 *| Copyright IBM Corp. 2009.  All Rights Reserved.                        |
 *|                                                                        |
 *| Note to US Government Users Restricted Rights: Use, duplication or     |
 *| disclosure restricted by GSA ADP Schedule Contract with IBM Corp.      |
 *+------------------------------------------------------------------------+
 */
#ifndef __UAL__RT__SERVICES__H__
#define __UAL__RT__SERVICES__H__
#include <string.h>

/**/
#ifdef WIN32
#ifndef strcasecmp
#define strcasecmp stricmp
#endif
#ifndef strcasencmp
#define strcasencmp strincmp
#endif
#endif


/*----- UALCore::String APIs to RTString APIs mapping functions (Begin) -----*/
	//
	// Usage:           char x = s.charAt(i);
	// Translated code: char x = charAtRTString(s, i);
	//
	inline char charAtRTString(const RTString *str, int index) {
		if (0 <= index  && index < strlen(str->Contents))
			return str->Contents[index];
		// throw new ArrayIndexOutOfBoundsException();
		return '\0';
    }

	//
	// Usage:			int i = s.indexOf(chs, index);
	// Translated code: int i = indexOfRTString(s, chs, index);
	//
	inline int indexOfRTString(const RTString *str, char ch, int fromIndex) {
		if (fromIndex < 0)
			fromIndex = 0;
		if (str == NULL || str->Contents == NULL ||
			(fromIndex > strlen(str->Contents)))
			return -1;

    	char *sub = strchr(&(str->Contents[fromIndex]), ch);
		if (sub == NULL)
			return -1;
    	return ( sub - str->Contents);
    }

	//
	// Usage:			int i = s.indexOf(chs);
	// Translated code: int i = indexOfRTString(s, chs);
	//
    inline int indexOfRTString(const RTString *str, char ch) {
    	return indexOfRTString(str, ch, 0);
    }

	//
	// Usage:			int i = s.indexOf(str, index);
	// Translated code: int i = indexOfRTString(s, str, index);
	//
	inline int indexOfRTString(const RTString *str, const RTString * subStr, int fromIndex) {
		if ( str == NULL || str->Contents == NULL ||
			 subStr == NULL || subStr->Contents == NULL)
			return -1;
		if (fromIndex < 0)
			fromIndex = 0;
		if (fromIndex + strlen(subStr->Contents) > strlen (str->Contents))
			return -1;
    	char *sub = strstr(&(str->Contents[fromIndex]), subStr->Contents);
		if (sub == NULL)
			return -1;
    	return ( sub - str->Contents);
    }

	//
	// Usage:			int i = s.indexOf(str);
	// Translated code: int i = indexOfRTString(s, str);
	//
	inline int indexOfRTString(const RTString *str, const RTString * substr) {
		return indexOfRTString(str, substr, 0);
    }

	//
	// Usage:			int c = s.compareTo(other);
	// Translated code:	int c = compareToRTString(s, other);
	//
    inline int compareToRTString(const RTString *str, const RTString *other) {
    	if (str != NULL && other != NULL ) {
    		if (str->Contents != NULL && other->Contents != NULL)
    			return strcmp(str->Contents, other->Contents);
    	}
    	//throw new Exception(); NPE?
    	return -1;
    }

	//
	// Usage:			int c = s.compareToIgnoreCase(other);
	// Translated code:	int c = compareToIgnoreCaseRTString(s, other);
	//
    inline int compareToIgnoreCaseRTString(const RTString *str, const RTString *other) {
    	if (str != NULL && other != NULL) {
    		if (str->Contents != NULL && other->Contents != NULL)
	    		return strcasecmp(str->Contents, other->Contents);
    	}
    	// throw new Exception(); NPE?
    	return -1;
    }

	//
	// Usage:			String result = s1.concat("hello");
	// Translated code: RTString * result = concatRTString(s1, "hello");
	//
    inline RTString *concatRTString(const RTString *str, const char *other) {
    	if (other != NULL) {
    		int otherLength = strlen (other);
    		if (otherLength > 0 ) {
    			if (str != NULL && str->Contents != NULL) {
    				otherLength += strlen(str->Contents);
    			}
    			char * buffer = new char[otherLength + 1];
    			buffer[0] = '\0';
    			if (str != NULL && str->Contents != NULL) {
    				strcpy(buffer, str->Contents);
    			}
    			buffer = strcat(buffer,other);
    			return new RTString(buffer);
    		}
    	}
    	return const_cast<RTString *>(str);
    }

	//
	// Usage:			String result = s1.concat(s2);
	// Translated code:	RTString *result = concatRTString(s2);
	//
    inline RTString *concatRTString(const RTString *str, const RTString *other) {
    	if (other != NULL)
    		return concatRTString(str, other->Contents);
    	return const_cast<RTString *>(str);
    }

	//
	// Usage:			int i = s.lastIndexOf(chs, index);
	// Translated code: int i = lastIndexOfRTString(s, chs, index);
	//
	inline int lastIndexOfRTString(const RTString *str, char ch, int fromIndex) {
		if (fromIndex < 0 )
			return -1;
		if (str != NULL && str->Contents != NULL )
		{
			int length = strlen(str->Contents);
			if (fromIndex >= length)
				fromIndex = length -1;
	    	char *sub = strrchr(&(str->Contents[fromIndex]), ch);
			if (sub != NULL)
				return ( sub - str->Contents);
		}
		return -1;
    }

	//
	// Usage:			int result = s1.lastIndexOf(c);
	// Translated code:	int result = lastIndexOfRTString(s1, c);
	//
    inline int lastIndexOfRTString(const RTString *str, char ch) {
		return lastIndexOfRTString(str, ch, 0);
    }

	//
	// Usage:			int i = s.lastIndexOf(str, index);
	// Translated code: int i = lastIndexOfRTString(s, str, index);
	//
	inline int lastIndexOfRTString(const RTString *str, const RTString * substr, int fromIndex) {
		if (fromIndex < 0)
			return -1;
		if (str != NULL && str->Contents != NULL) {
			int length = strlen(str->Contents);
			if (fromIndex >= length)
				fromIndex = length -1;
			char *sub  = &(str->Contents[fromIndex]);
			char *sub1  = sub;
			char *temp = NULL;
			while ((sub = strstr(sub1, substr->Contents)) != NULL) {
				temp = sub;
				sub1 = ++sub;
				if (*sub == '\0')
					break;
			}
			if (temp == NULL)
				return -1;
			return ( temp - str->Contents);
		}
    }

	//
	// Usage:			int i = s.lastIndexOf(str);
	// Translated code: int i = lastIndexOfRTString(s, str);
	//
	inline int lastIndexOfRTString(const RTString *str, const RTString * substr) {
		return lastIndexOfRTString(str, substr, 0);
    }

	//
	// Usage:			int length = s1.length();
	// Translated code:	int length = lengthRTString(s1);
	//
    inline int lengthRTString(const RTString *str) {
    	if (str == NULL || str->Contents == NULL)
    		return 0;
    	return strlen(str->Contents);
    }

/*----- UALCore::String APIs to RTString APIs mapping functions (End) -----*/
#endif
