/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM                                   |
 *| Copyright IBM Corp. 2009.  All Rights Reserved.                        |
 *|                                                                        |
 *| Note to US Government Users Restricted Rights: Use, duplication or     |
 *| disclosure restricted by GSA ADP Schedule Contract with IBM Corp.      |
 *+------------------------------------------------------------------------+
 */
#ifndef __UAL__SERVICES__H__
#define __UAL__SERVICES__H__

/* includes */
#include <iostream>
#include <string>
#include <exception>
#include <stdexcept>
#include <cassert>

// temporary include for RTString
#include <UALRTServices.h>

#ifdef __MINGW32__
// Socket
#include <winsock2.h>
// Thread
#include <windows.h>
#include <process.h>
#include <tchar.h>
#include <Windows.h>
#else
extern "C" {
#include <fcntl.h>
#include <errno.h>
#include <unistd.h>
#include <cstdio>
#include <sys/types.h>
#include <cstring>
#include <stdio.h>

// Stream
#include <istream>
#include <ostream>
#include <streambuf>
// Socket
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
// Thread
#include <pthread.h>
}
#endif

/****Section: Macros****/

// Common Socket Macro definitions to Win32 APIs and ANSI APIs


#define FN_SOCKET(af, type, protocol) 			::socket((af), (type), (protocol))
#define FN_CONNECT(sockFd, addr, addrLen) 		::connect((sockFd), (addr), (addrLen))
#define FN_BIND(sockFd, addr, addrLen) 			::bind((sockFd), (addr), (addrLen))
#define FN_LISTEN(sockFd, backLogs) 			::listen((sockFd), (backLogs))
#define FN_ACCEPT(sockFd, addr, addrLen)		::accept((sockFd), (addr), (addrLen))
#define FN_GETSOCKNAME(sockFd, addr, addrLen)	::getsockname((sockFd), (addr), (addrLen))
#define FN_GETHOSTBYNAME(name)					::gethostbyname((name))
#define FN_GETNAMEINFO(sa, saLen, host, hostLen, serv, servLen, flags) \
												::getnameinfo((sa), (saLen), (host), (hostLen), (serv), (servLen), (flags))
#define FN_SEND(socketFd, ca, length, flags)	::send((socketFd), (ca), (length), (flags))
#define FN_RECEIVE(socketFd, ca, length, flags)	::recv((socketFd), (ca), (length), (flags))
#ifndef DEFAULT_BACKLOGS
#define DEFAULT_BACKLOGS 6
#endif

#if defined(__MINGW32__)

// Socket Win32 related macros/typedefs
typedef int socklen_t;
typedef int sa_family_t;
typedef int in_addr_t;

#define FN_CLOSE_SOCKET(sockFd)					closesocket((sockFd))
#define LOCALHOST 								""
#define SOCKET_STARTUP() \
	WORD wVersionRequested; \
	WSADATA wsaData; \
	int wsaerr; \
	wVersionRequested = MAKEWORD(2, 2); \
	if((wsaerr = ::WSAStartup(wVersionRequested, &wsaData))) { \
		throw std::ios_base::failure("WSAStartup failed: The Winsock dll NOT found !"); \
	}
#define SOCKET_CLEANUP() 		WSACleanup();

// I/O related macros/typedefs
#define FN_OPEN_READ(fileName) \
	CreateFile((fileName), FILE_READ_DATA, 0, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL)
#define FN_OPEN_WRITE(fileName) \
	CreateFile((fileName), FILE_WRITE_DATA, 0, NULL, CREATE_NEW, FILE_ATTRIBUTE_NORMAL, NULL)
#define FN_OPEN_APPEND(fileName) \
	CreateFile((fileName), FILE_APPEND_DATA, 0, NULL, CREATE_NEW, FILE_ATTRIBUTE_NORMAL, NULL)
#define FN_WRITE(fd, ba, len, lenWritten) \
	WriteFile((fd), (ba), (len), (lenWritten), NULL)
#define FN_READ(fd, ba, len, lenRead) \
	ReadFile((fd), (ba), (len), (lenRead), NULL)
#define FN_CLOSE(fd)			CloseHandle((fd))

// Thread Win32 related macros
// generic macros
#define THREAD_CREATION_ERROR(retVal)	((retVal) == NULL)
#define GET_ERROR_CODE(val)				GetLastError()

// types/macros
#define T_SOCKET_DESCRIPTOR 			SOCKET
#define T_IO_RET_TYPE					DWORD
#define T_HANDLE						HANDLE
#define T_ATTR_TYPE						int			// Dummy
#define T_THREAD_ID			 			unsigned
#define T_THR_FN_RET_TYPE				unsigned

// functions macros
#define FN_THREAD_ATTR_INIT(attr)			// Do Nothing
#define FN_THREAD_ATTR_DESTROY(attr)		// Do Nothing
#define FN_THREAD_EXIT(param)			_endthreadex((param))
#define FN_GET_ID()						threadId
#define FN_THREAD_CREATE(thrId, attr, thrFn, thrParam) \
	(HANDLE)(_beginthreadex(NULL, 0, (thrFn), (thrParam), 0, (thrId)))
#define FN_THREAD_JOIN(handle, id, ptr) WaitForSingleObject((handle), INFINITE)
#define FN_SLEEP(millis)				Sleep((millis))

#else /* NOT __MINGW32__ */

// Socket ANSI related macros/typedefs
#define T_SOCKET_DESCRIPTOR 				int
#define SOCKET_STARTUP()		 			// Dummy
#define SOCKET_CLEANUP() 					// Dummy
#define INVALID_SOCKET 						-1
#define INVALID_HANDLE_VALUE				-1
#define LOCALHOST "localhost"
#define FN_CLOSE_SOCKET(sockFd)				::close((sockFd))

// I/O related macros/typedefs
#define FN_OPEN_READ(fileName) \
	::open((fileName), O_RDONLY, 0)
#define FN_OPEN_WRITE(fileName) \
	::open((fileName), O_WRONLY | O_CREAT, 0644)
#define FN_OPEN_APPEND(fileName) \
	::open((fileName), O_WRONLY | O_APPEND | O_CREAT, 0644)
#define FN_WRITE(fd, ba, len, lenWritten) \
	(*(lenWritten)) = ::write((fd), (ba), (len))
#define FN_READ(fd, ba, len, lenRead) \
	(*(lenRead)) = ::read((fd), (ba), (len))
#define FN_CLOSE(fd) \
	::close((fd))

// Thread ANSI related macros/typedefs
// generic macros
#define THREAD_CREATION_ERROR(retVal)		((retVal) != 0)
#define GET_ERROR_CODE(val)					(val)

// types macros
#define T_IO_RET_TYPE						int
#define T_HANDLE							int
#define	T_ATTR_TYPE							pthread_attr_t
#define T_THREAD_ID							pthread_t
#define T_THR_FN_RET_TYPE					void *

// functions macros
#define FN_THREAD_ATTR_INIT(attr) \
	::pthread_attr_init((attr))
#define FN_THREAD_ATTR_DESTROY(attr) \
	::pthread_attr_destroy((attr))
#define FN_THREAD_EXIT(param) \
	::pthread_exit((param))
#define FN_GET_ID()	\
	threadHandle

#define FN_THREAD_CREATE(thrId, attr, thrFn, thrParam) \
	::pthread_create((thrId), (attr), (thrFn), (thrParam))
#define FN_THREAD_JOIN(handle, id, ptr) \
	::pthread_join((id), NULL)
#define FN_SLEEP(millis) \
	::usleep((useconds_t) millis * 1000L);
#define WINAPI						// Dummy
#endif /* NOT __MINGW32__ */

/****Exception Section****/

// In Java UnkownHostException extends from IOException
// I have just extended from exception in C++ because I
// couldn't see mapping to ios_base::failure

class UnknownHostException: public std::exception {
public:
	UnknownHostException() {
	}
};

class IllegalThreadStateException: public std::exception {
private:
	long errNo;

public:
	IllegalThreadStateException(long errValue) {
		errNo = errValue;
	}

	virtual const char* what() const throw() {
		if (errNo == EAGAIN) {
			return "The process lacks the resources to create another thread or the total number of threads in a process would exceed\n";
		} else if (errNo == EINVAL) {
			return "Invalid \"attr\" parameter\n";
		} else {
			std::cerr << "errno = " << errNo;
			return "Unknown Error";
		}
	}

	virtual int getErrorNumber() const {
		return errno;
	}
};

class InterruptedException: public std::exception {

};

/****Section: Stream Implementation****/

// TODO: In Progress - should be abstract I guess (14-12-09)
class InputStream /* : public std::istream */{
protected:
	std::istream& is;
private:
	// 	 Not implementing the skipBuffer impl.
	//	 static const int SKIP_BUFFER_SIZE = 2048;
	//	 static char *skipBuffer;
public:

	// TODO: default constructor should rightly initialize is
	InputStream() :
		is(std::cin) {
	}

	InputStream(std::istream& isNew) :
		is(isNew) {
	}

	virtual ~InputStream() {
	}

	virtual int read() const throw(std::ios_base::failure) = 0;

	virtual int read(char ca[], int off, int len) const
				throw(std::ios_base::failure) = 0;

	virtual long skip(long noOfBytesToSkip) throw(std::ios_base::failure) {

		// TODO: check if noOfBytesToSkip is > total bytes to read - if poss.
		is.seekg(noOfBytesToSkip, std::ios::cur);
		return noOfBytesToSkip;
	}

	void close() throw(std::ios_base::failure) {
	}
};

// TODO: In Progress - should be abstract I guess (14-12-09)
class OutputStream /* : public std::ostream */{
protected:
	std::ostream & os;
public:

	// TODO: default constructor should rightly initialize os
	OutputStream() :
		os(std::cout) {
	}

	OutputStream(std::ostream& osNew) :
		os(osNew) {
	}

	virtual ~OutputStream() {
	}

	virtual void write(int b) throw(std::ios_base::failure) = 0;

	virtual void write(char ca[], int off, int len) throw(std::ios_base::failure) = 0;

	virtual void flush() {
	}
	virtual void close() throw(std::ios_base::failure) {
	}
};

class FileInputStream: public InputStream {
protected:
	T_HANDLE fd;

private:
	void init(const char * fileName) throw(std::ios_base::failure) {
		// TODO
		fd = FN_OPEN_READ(fileName);
		if (fd == INVALID_HANDLE_VALUE) {
			throw std::ios_base::failure("Error in opening file for reading");
		}
	}

public:
	FileInputStream(T_HANDLE fdNew) :
		fd(fdNew) {
	}
/*
	FileInputStream(std::string fileName) throw(std::ios_base::failure) {
		init(fileName.c_str());
	}
*/
	// temporary as String converts to RTString
	FileInputStream(RTString fileName) throw(std::ios_base::failure) {
		//std::cout << "--> FileInputStream(RTString* fileName)--> "<< fileName.Contents << "\n";
		init(fileName.Contents);
	}

	virtual ~FileInputStream() {
		close();
		fd = INVALID_HANDLE_VALUE;
	}

	virtual int read() const throw(std::ios_base::failure) {
		char ca[1];
		read(ca, 0, 1);
		return ca[0];
	}

	virtual int read(char *ca, int off, int len) const throw(std::ios_base::failure) {
		//std::cout << "FileInputStream::read\n";
		if ((ca == NULL) || (len == 0)) {
			return 0;
		}
		T_IO_RET_TYPE lenRead;
		FN_READ(fd, ca+off, len, &lenRead);
		if (lenRead < 0)
			throw std::ios_base::failure("Error in syscall \'read\'");
		return lenRead;
	}

	virtual long skip(long noOfBytesToSkip) throw(std::ios_base::failure) {

		// TODO: check if noOfBytesToSkip is > total bytes to read - if poss.
		return 0;
	}

	virtual void close() throw(std::ios_base::failure) {
		FN_CLOSE(fd);
	}
};

class FileOutputStream: public OutputStream {
protected:
	T_HANDLE fd;

private:
	void init(const char * fileName, bool append) {
//		static int i = 0;
//		++i;
//		std::cout<< "oooo>> FileOutputStream::init --> " << i << ":" <<fileName << "\n";
//		fflush(stdout);
		if (append) {
			fd = FN_OPEN_APPEND(fileName);
		} else {
			fd = FN_OPEN_WRITE(fileName);
		}
		//std::cout << "fd = " << fd << "\n";
		//fflush(stdout);
		if (fd == INVALID_HANDLE_VALUE) {
//			std::cout << "oooo> Error - INVALID_HANDLE_VALUE " << i << "fd = " << fd << "\n";
			throw std::ios_base::failure("Error in opening file for reading");
		}
	}
public:
	FileOutputStream(T_HANDLE fdNew) :
		fd(fdNew) {
	}
/*
	FileOutputStream(std::string fileName) throw(std::ios_base::failure) {
		init(fileName.c_str(), false);
	}

	FileOutputStream(std::string fileName, bool append)
			throw(std::ios_base::failure) {
		init(fileName.c_str(), append);
	}
*/
	// temporary as String converts to RTString
	FileOutputStream(RTString fileName) throw(std::ios_base::failure) {
		//std::cout << "--> FileOutputStream(RTString * fileName)\n";
		try {
			init(fileName.Contents, false);
		} catch(std::ios_base::failure e) {
			std::cout << "!!! Error in creating file !!!\n";
			throw e;
		}
	}

	FileOutputStream(RTString  fileName, bool append)
			throw(std::ios_base::failure) {
		init(fileName.Contents, append);
	}

	virtual ~FileOutputStream() {
		close();
	}

	virtual void write(int b) throw(std::ios_base::failure) {
		char buff[2] = { 0 };
		buff[0] = (char) b;
		write(buff, 0, 1);
	}

	virtual void write(char buf[], int off, int len)
			throw(std::ios_base::failure) {
		T_IO_RET_TYPE nBytes;

		FN_WRITE(fd, buf + off, len, &nBytes);
		//std::cout << "write: " << nBytes << "\n";
		if (nBytes < 0) {
			throw std::ios_base::failure("call to syscall \'write\' failed");
		}
	}

	virtual void close() throw(std::ios_base::failure) {
		// TODO
	}
};

/****Section: Socket Implementation****/


class SocketInputStream: public InputStream {
protected:
	T_SOCKET_DESCRIPTOR fd;

public:
	SocketInputStream(T_SOCKET_DESCRIPTOR fdNew) :
		fd(fdNew) {
	}

	virtual ~SocketInputStream() {
		close();
	}

	virtual int read() const throw(std::ios_base::failure) {
		char ca[1];
		read(ca, 0, 1);
		return ca[0];
	}

	virtual int read(char *ca, int offset, int len) const
			throw(std::ios_base::failure) {
		if ((ca == NULL) || (len == 0)) {
			return 0;
		}
		//std::cout << "SocketInputStream::read\n";
		int lenRead = FN_RECEIVE(fd, ca + offset, len, 0);
		if (lenRead < 0)
			throw std::ios_base::failure("Error in syscall \'receive\'");
		return lenRead;
	}

	virtual long skip(long noOfBytesToSkip) throw(std::ios_base::failure) {

		// TODO: check if noOfBytesToSkip is > total bytes to read - if poss.
		return 0;
	}

	virtual void close() throw(std::ios_base::failure) {
		// TODO
	}
};

class SocketOutputStream: public OutputStream {
protected:
	T_SOCKET_DESCRIPTOR fd;

public:
	SocketOutputStream(T_SOCKET_DESCRIPTOR fdNew) :
		fd(fdNew) {
	}

	virtual ~SocketOutputStream() {
		close();
		fd = INVALID_SOCKET;
	}

	virtual void write(int b) throw(std::ios_base::failure) {
		char buff[2] = { 0 };
		buff[0] = (char) b;
		write(buff, 0, 1);
	}

	virtual void write(char buf[], int offset, int len)
			throw(std::ios_base::failure) {
		int nBytes = 0;
		//std::cout << "SocketOutputStrem::write\n";
		nBytes = FN_SEND(fd, buf+offset, len, 0);
		if (nBytes < 0) {
			throw std::ios_base::failure("call to syscall \'send\' failed");
		}
	}

	virtual void close() throw(std::ios_base::failure) {
		// TODO
	}
};

class InetAddress {

private:
	std::string hostName;
	hostent *hp;
	in_addr_t address;
	int family;

	// private constructor - InetAddress cannot be derived
	InetAddress(std::string host) :
		hostName(host) {
		init();
	}

	void init() throw (std::ios::ios_base::failure) {
		SOCKET_STARTUP();
		const char * hostStr = hostName.empty() ? NULL : hostName.c_str();
		if ((hp = FN_GETHOSTBYNAME(hostStr)) == NULL) {
			std::string exceptionStr = "gethostname failed for ";
			exceptionStr += hostName;
			throw std::ios_base::failure(exceptionStr);
		}

		address = ((in_addr *) (hp->h_addr))->s_addr;
		family = AF_INET;
		return;
	}

public:
	virtual ~InetAddress() {
		/* hp should not be freed; read man page of gethostbyname for more details */
		SOCKET_CLEANUP();
	}

	static InetAddress *getByName(std::string host)
			throw (UnknownHostException) {
		return new InetAddress(host);
	}

	static InetAddress *getLocalHost() throw (UnknownHostException) {
		return new InetAddress("localhost");
	}

	in_addr_t getFamily() {
		return family;
	}

	std::string getHostName() {
		return hostName;
	}

	int getAddress() {
		return address;
	}
};

class Socket {
private:
	bool closed;
	bool created;
	bool connected;
	int port;
	int localPort;
	T_SOCKET_DESCRIPTOR socketFd;
	InetAddress * inetAddr;
	InetAddress * localAddr;

	sockaddr_in sockAddr;

	void init(InetAddress *addr, int port) throw (std::ios_base::failure,
			std::invalid_argument) {
		localPort = -1;
		localAddr = (InetAddress *) 0;

		SOCKET_STARTUP();

		if (port < 0 || port > 0xFFFF) {
			char exceptionStr[32];
			sprintf(exceptionStr, "invalid port: %d", port);
			throw std::invalid_argument(exceptionStr);
		}

		this->port = port;
		inetAddr = addr;
		if (inetAddr == NULL) {
			inetAddr = InetAddress::getLocalHost();
		}

		memset(&sockAddr, 0, sizeof(sockAddr));

		sockAddr.sin_family = inetAddr->getFamily();
		sockAddr.sin_addr.s_addr = inetAddr->getAddress();

		sockAddr.sin_port = htons(port);

		if ((socketFd = FN_SOCKET(AF_INET, SOCK_STREAM, 0)) == INVALID_SOCKET) {
			SOCKET_CLEANUP();
			throw std::ios_base::failure("socket");
		}

		created = true;

		if (FN_CONNECT(socketFd, (sockaddr *) &sockAddr, sizeof(sockAddr))) {
			char exceptionStr[256];
			sprintf(exceptionStr, "connect failed for %s:%d",
					(inetAddr->getHostName()).c_str(), this->getPort());
			SOCKET_CLEANUP();
			throw std::ios_base::failure(exceptionStr);
		}
		connected = true;

		// update the localAddress and localPort
		getLocalAddress();
	}

	bool isConnected() const {
		return connected;
	}

public:

	// for dummy Socket use in ServerSocket.accept()
	Socket(T_SOCKET_DESCRIPTOR newSockFd) throw (std::ios_base::failure) :
		closed(false), created(false), socketFd(newSockFd), inetAddr(
				(InetAddress *) 0), localAddr((InetAddress *) 0) {
	}

	// Maps to UAL's following function:
	// Socket.Socket(InetAddress *host, int port);
	Socket(InetAddress *address, int port) throw (std::ios_base::failure) :
		closed(false), created(false), socketFd(INVALID_SOCKET) {
		init(address, port);
	}

	virtual ~Socket() {
		if (localAddr) {
			delete (localAddr);
			localAddr = NULL;
			localPort = -1;
		}
		if (inetAddr) {
			delete (inetAddr);
		} SOCKET_CLEANUP();
	}

	// Maps to UAL's following function:
	// public InetAddress Socket.getInetAddress();
	virtual InetAddress *getInetAddress() {
		return inetAddr;
	}

	// Maps to UAL's following function:
	// public int Socket.getPort();
	virtual int getPort() const {
		if (!isConnected())
			return -1;
		return ntohs(sockAddr.sin_port);
	}

	// Maps to UAL's following function:
	// public int Socket.getlocalAddress();
	virtual InetAddress * getLocalAddress() {

		sockaddr_in tmpSockAddr;

		socklen_t tmpSockAddrLen = sizeof(tmpSockAddr);

		if (FN_GETSOCKNAME(socketFd, (sockaddr *) &tmpSockAddr, &tmpSockAddrLen)) {
			throw std::ios_base::failure("getsockname failed");
		}

		localPort = ntohs(tmpSockAddr.sin_port);
		//char hostName[256] = { 0, };
		//char serviceName[256] = { 0, };
		//		if (FN_GETNAMEINFO((sockaddr *) &tmpSockAddr, tmpSockAddrLen, hostName,
		//				256, serviceName, 256, 0)) {
		//			localAddr = InetAddress::getLocalHost();
		//		} else {
		//			localAddr = InetAddress::getByName(hostName);
		//		}
		// TODO: the below line is tmp fix for above 6 lines
		// commented. Right now cygwin doesn't have the api getnameinfo
		localAddr = InetAddress::getLocalHost();
		return localAddr;
	}

	// Maps to UAL's following function:
	// public int Socket.getLocalPort();
	virtual int getLocalPort() {
		if (localPort != -1)
			return localPort;
		getLocalAddress();
		return localPort;
	}

	// Maps to UAL's following function:
	// public InputStream Socket.getInputStream()
	virtual InputStream* getInputStream() throw(std::ios_base::failure) {
		InputStream * sis = new SocketInputStream(socketFd);
		return sis;
	}

	// Maps to UAL's following function:
	// public OutputStream Socket.getOutputstream()
	virtual OutputStream* getOutputStream() throw(std::ios_base::failure) {
		OutputStream * sos = new SocketOutputStream(socketFd);
		return sos;
	}

	// Maps to UAL's following function:
	// public int Socket.close()
	virtual void close() throw (std::ios_base::failure) {
		if (!isClosed()) {
			if (created) {
				if (FN_CLOSE_SOCKET(socketFd)) {
					SOCKET_CLEANUP();
					throw std::ios_base::failure("close syscall failed");
				}
			}
			closed = true;
		}
	}

	// Maps to UAL's following function:
	// public int Socket.isClosed()
	virtual bool isClosed() const {
		return closed;
	}

	// <tentative> : for PBX purpose only
	virtual int read(char ca[], int offset, int len)
			throw(std::ios_base::failure) {
		int lenRead = 0;
		if ((ca == NULL) || (len == 0)) {
			return 0;
		}
		lenRead = FN_RECEIVE(socketFd, ca + offset, len, 0);
		if (lenRead < 1)
			throw std::ios_base::failure("Error in syscall \'read\'");
		return lenRead;
	}

	virtual void write(char buf[], int offset, int len)
			throw(std::ios_base::failure) {
		int nBytes = 0;

		nBytes = FN_SEND(socketFd, buf+offset, len, 0);
		if (nBytes < 0) {
			throw std::ios_base::failure("call to syscall \'write\' failed");
		}
	}
};

class ServerSocket {
private:
	T_SOCKET_DESCRIPTOR socketFd;
	bool closed;
	bool bound;
	bool created;
	int port;
	InetAddress * inetAddr;
	sockaddr_in servSockAddr;
	sockaddr clientSockAddr;
	socklen_t cAddrLen;

	void init(int port, int backlog, InetAddress *addr)
			throw (std::ios_base::failure, std::invalid_argument) {

		socketFd = INVALID_SOCKET;
		created = false;
		bound = false;
		closed = false;

		SOCKET_STARTUP();
		if (port < 0 || port > 0xFFFF) {
			char exceptionStr[32];
			sprintf(exceptionStr, "invalid port: %d", port);
			throw std::invalid_argument(exceptionStr);
		}

		this->port = port;
		inetAddr = addr;

		if (inetAddr == NULL) {
			inetAddr = InetAddress::getLocalHost();
		}

		memset(&servSockAddr, 0, sizeof(servSockAddr));

		servSockAddr.sin_family = inetAddr->getFamily();
		servSockAddr.sin_addr.s_addr = inetAddr->getAddress();

		servSockAddr.sin_port = htons(port);

		if ((socketFd = FN_SOCKET(AF_INET, SOCK_STREAM, 0)) == INVALID_SOCKET) {
			SOCKET_CLEANUP();
			throw std::ios_base::failure("socket syscall failed");
		}

		created = true;

		if (backlog < 1)
			backlog = DEFAULT_BACKLOGS;

		if (FN_BIND(socketFd, (sockaddr*) &servSockAddr, sizeof(servSockAddr))) {
			SOCKET_CLEANUP();
			throw std::ios_base::failure("bind syscall failed");
		}
		bound = true;
		if (FN_LISTEN(socketFd, backlog) < 0) {
			SOCKET_CLEANUP();
			throw std::ios_base::failure("listen syscall failed");
		}
	}

	bool isBound() const {
		return bound == true;
	}

public:

	// Maps to UAL's following function:
	// ServerSocket.ServerSocket(int port) throws IOException
	ServerSocket(int port) throw (std::ios_base::failure,
			std::invalid_argument) {
		init(port, DEFAULT_BACKLOGS, NULL);
	}

	// Maps to UAL's following function:
	// ServerSocket.ServerSocket(int port, int backlog) throws IOException;
	ServerSocket(int port, int backlog) throw (std::ios_base::failure,
			std::invalid_argument) {
		init(port, backlog, NULL);
	}

	// Maps to UAL's following function:
	// ServerSocket.ServerSocket(int port, int backlog, InetAddress bindAddr) throws IOException;
	ServerSocket(int port, int backlog, InetAddress *bindAddr)
			throw (std::ios_base::failure, std::invalid_argument) {
		// ## in model this one doesn't throw any exception
		init(port, backlog, bindAddr);
	}

	virtual ~ServerSocket() {
		if (inetAddr) {
			delete (inetAddr);
		} SOCKET_CLEANUP();
	}

	// Maps to UAL's following function:
	// Socket ServerSocket.accept() throws IOException;
	virtual Socket *accept() throw (std::ios_base::failure) {
		T_SOCKET_DESCRIPTOR newSockFd = INVALID_SOCKET;
		//std::cout << "ServerSocket.accept()\n";
		memset(&clientSockAddr, 0, sizeof(clientSockAddr));
		cAddrLen = sizeof(sockaddr);
		if ((newSockFd
				= FN_ACCEPT(socketFd, (sockaddr *) &clientSockAddr, &cAddrLen))
				== INVALID_SOCKET) {
			throw std::ios_base::failure("accept system call failed");
		}
		// now we have server finally up and running, listening to accept, so
		return new Socket(newSockFd);
	}

	// Maps to UAL's following function:
	// ServerSocket.void close() throws IOException;
	virtual void close() throw (std::ios_base::failure) {
		if (!isClosed()) {
			if (created) {
				if (FN_CLOSE_SOCKET(socketFd) == -1) {
					throw std::ios_base::failure("close");
				}
			}
			closed = true;
		}
	}

	// Maps to UAL's following function:
	// public boolean ServerSocket.isClosed();
	virtual bool isClosed() const {
		return closed;
	}

	// Maps to UAL's following function:
	// public int ServerSocket.getLocalPort();
	virtual int getLocalPort() const {
		return (isBound()) ? ntohs(servSockAddr.sin_port) : -1;
	}

	// Maps to UAL's following function:
	// public int ServerSocket.getInetAddress();
	virtual InetAddress *getInetAddress() {
		return inetAddr;
	}
};

/****Section: Thread Implementation****/

// abstract class Runnable
class Runnable {
public:
	virtual void run() = 0;
};

class Thread: public Runnable {

private:
	// the below var isn't used yet but maybe used in later versions of Thread
	bool started;
	Runnable *runnable;
	std::string name;
	T_HANDLE threadHandle;
	T_ATTR_TYPE attr;
	T_THREAD_ID threadId;

	void initialize() {
		// startFn = &runnable->run();
		FN_THREAD_ATTR_INIT(&attr);
		threadHandle = 0; // some illegal value
		started = false;
	}

public:
	// Maps to UAL's following function:
	// Thread()
	Thread() :
		runnable(this) {
		initialize();
	}

	// Maps to UAL's following function:
	// Thread(Runnable target)
	Thread(Runnable *target) :
		runnable(target) {
		initialize();
	}

	// Maps to UAL's following function:
	// Thread(Runnable target, String name)
	Thread(Runnable *target, std::string newName) :
		runnable(target), name(newName) {
		initialize();
	}

	// Maps to UAL's following function:
	// Thread(String name)
	Thread(std::string newName) :
		runnable(this), name(newName) {
		initialize();
	}

	// destructor
	virtual ~Thread() {
		FN_THREAD_ATTR_DESTROY(&attr);
		FN_THREAD_EXIT(0);
	}

	// Maps to UAL's following function:
	// void run()
	virtual void run() {

	}

	// Maps to UAL's following function:
	// Thread.getId()  Not implemented in model yet -- TODO model changes
	virtual T_THREAD_ID getThreadId() {
		return FN_GET_ID();
	}

	// Maps to UAL's following function:
	// String getName()
	std::string getName() const {
		return name;
	}

	// Maps to UAL's following function:
	// void setName(String name);
	void setName(std::string name) {
		this->name = name;
	}

	// Maps to UAL's following function:
	// void start()
	virtual void start() throw(IllegalThreadStateException) {
		initialize();

		T_HANDLE returnVal;
		returnVal
				= FN_THREAD_CREATE(&threadId, &attr, Thread::threadStart, (void*) runnable);
		if (THREAD_CREATION_ERROR(returnVal)) {
			throw IllegalThreadStateException(GET_ERROR_CODE(returnVal));
		}
		started = true;
	}

	// Maps to UAL's following function:
	// void join() throws InterruptedException ;
	void join() throw(InterruptedException) {
		FN_THREAD_JOIN(threadHandle, threadId, NULL);
	}

	static void sleep(long millis) throw(InterruptedException) {
		// in the model its milliseconds but usleep accepts microseconds
		FN_SLEEP(millis);

		// The manpage of sleep/usleep talks about process but many forum say
		// that the POSIX manpage is broken.
		// TODO: Needs to checked out with a sample example
		// Possible alternatives:
		// 1. select(NULL, NULL, NULL, &tv) where tv is a struct timeval
		// 2. pthread_cond_timedwait()
	}

	static T_THR_FN_RET_TYPE WINAPI threadStart(void *thr) {
		Runnable *runnable = (Runnable *) thr;
		runnable->run();
		return 0;
	}
};

/*
 * Condition mechanism using the classes Condition and ConditionObject
 */
#ifdef __MINGW32__

// TODO

#else /* NOT __MINGW32__ */

class Condition {
protected:
	pthread_mutex_t *associatedMutex;
	pthread_cond_t condition;

	void initialize() {
		pthread_cond_init(&condition, NULL);
	}

public:
	// Maps to UAL's following function:
	// public Condition (); <-- notice the extra param in implementation
	Condition(pthread_mutex_t *mutex) :
		associatedMutex(mutex) {
		initialize();
	}

	// Maps to UAL's following function:
	virtual ~Condition() {
		pthread_cond_destroy(&condition);
	}

	// Maps to UAL's following function:
	// void await()
	virtual void await() = 0; //  throw(InterruptedException) ?

	//	TODO: These overloaded versions of await have not been implemented, can be
	//  implemented though with very less effort using pthread_cond_timedwait
	//
	//    virtual long awaitNanos(long nanosTimeout) throw(InterruptedException) = 0;
	//
	//    virtual bool await(long time, TimeUnit unit) throw(InterruptedException) = 0;
	//
	//    virtual bool awaitUntil(Date deadline) throw(InterruptedException) = 0;

	// Maps to UAL's following function:
	// void signal()
	virtual void signal() = 0;

	// Maps to UAL's following function:
	// void signalAll()
	virtual void signalAll() = 0;
};

class ConditionObject: public Condition {
private:
	void initialize() { // do nothing as of now
	}
public:

	// constructors:

	// Maps to UAL's following function:
	// public ConditionObject() <-- notice the extra param in implementation
	ConditionObject(pthread_mutex_t *mutex) :
		Condition(mutex) {
		initialize();
	}

	virtual ~ConditionObject() {
	}

	// Maps to UAL's following function:
	// void await()
	void await() {
		// TODO ? :throw(InterruptedException) - not handled in C++ implementation of await
		int retValue;
		if ((retValue = pthread_cond_wait(&condition, associatedMutex))) {
			// TODO: throw some exception ?
		}
	}

	// Maps to UAL's following function:
	// void signal()
	void signal() {
		pthread_cond_signal(&condition);
	}

	// Maps to UAL's following function:
	// void signalAll()
	void signalAll() {
		pthread_cond_broadcast(&condition);
	}
};

// Locking mechanism using the classes Lock and ReentrantLock
class Lock {
public:
	// Maps to UAL's following function:
	// void lock()
	virtual int lock() = 0;

	// Maps to UAL's following function:
	// boolean tryLock()
	virtual bool tryLock() = 0;

	// Maps to UAL's following function:
	// void unlock()
	virtual int unlock() = 0;

	// Maps to UAL's following function:
	// Condition newCondition()
	virtual Condition* newCondition() = 0;
};

class ReentrantLock: public Lock {
	// implements Serializable
private:
	bool locked;
	pthread_mutex_t mutex;
	friend class Condition;
public:

	// constructors:
	// Maps to UAL's following function:
	ReentrantLock() :
		locked(false) {
		pthread_mutex_init(&mutex, NULL);
	}

	// destructors:
	// Maps to UAL's following function:
	// void ReentrantLock()
	virtual ~ReentrantLock() {
		pthread_mutex_destroy(&mutex);
	}

	// Maps to UAL's following function:
	// void lock()
	virtual int lock() {
		int retVal = pthread_mutex_lock(&mutex);
		locked = true;
		return retVal;
	}

	// Maps to UAL's following function:
	// boolean tryLock()
	virtual bool tryLock() {
		return pthread_mutex_trylock(&mutex) == 0;
	}

	// Maps to UAL's following function:
	// void unlock()
	virtual int unlock() {
		int retVal = pthread_mutex_unlock(&mutex);
		locked = false;
		return retVal;
	}

	// Maps to UAL's following function:
	// Condition newCondition()
	// ?? Can there be more than one condition for the same mutex. Find out ?
	virtual Condition* newCondition() {
		// should assert or throw exception ? assert for now(onus on user)
		assert(locked);
		return new ConditionObject(&mutex);
	}
};

#endif /* NOT __MINGW32__ */

#endif /* __UAL__SERVICES__H__ */
